@@ -4,6 +4,7 @@
 # @Purpose :
 import os
 import json
+import numpy as np
 from datetime import datetime
 from collections.abc import Callable
 
@@ -82,6 +83,8 @@ class XtMdApi:
 
         self.available_exchange: list[Exchange] = []
 
+        self.accu_volumes: dict[str, int] = {}
+
     def onMarketData(self, data: dict) -> None:
         """行情推送回调"""
 
@@ -140,16 +143,36 @@ class XtMdApi:
 
             if tick.vt_symbol in symbol_limit_map:
                 tick.limit_up, tick.limit_down = symbol_limit_map[tick.vt_symbol]
+
+            # 判断收盘状态
+            tick.extra = {
+                "market_closed": False,
+            }
+
+            if contract.product == Product.ETF:
+                tick.extra["iopv"] = d.get("pe", np.nan)
+
+            # 非衍生品可以通过openInt字段判断证券状态
+            if contract.product not in {Product.FUTURES, Product.OPTION}:
+                tick.extra["market_closed"] = d["openInt"] == 15
+            # 衍生品该字段为持仓量，需要通过结算价判断
+            elif d["settlementPrice"] > 0:
+                tick.extra["market_closed"] = True
+
             return tick
 
         for xt_symbol, buf in data.items():
             if isinstance(buf, dict):
                 tick = parse_data_dict(xt_symbol, buf)
+                tick.last_volume = max(0, tick.volume - self.accu_volumes.get(tick.vt_symbol, 0))
                 self.gateway.on_tick(tick)
+                self.accu_volumes[tick.vt_symbol] = tick.volume
             elif isinstance(buf, list):
                 for d in buf:
                     tick = parse_data_dict(xt_symbol, d)
+                    tick.last_volume = max(0, tick.volume - self.accu_volumes.get(tick.vt_symbol, 0))
                     self.gateway.on_tick(tick)
+                    self.accu_volumes[tick.vt_symbol] = tick.volume
 
     def connect(
         self,
@@ -280,6 +303,7 @@ class XtMdApi:
         for xt_symbol in xt_symbols:
             # 筛选需要的合约
             product = None
+            vol_step = 100
             symbol, xt_exchange = xt_symbol.split(".")
 
             if xt_exchange == "SZ":
@@ -289,6 +313,7 @@ class XtMdApi:
                     product = Product.ETF
                 else:
                     product = Product.INDEX
+                    vol_step = 1
             elif xt_exchange == "SH":
                 if xt_symbol.startswith(("60", "68")):
                     product = Product.EQUITY
@@ -296,6 +321,7 @@ class XtMdApi:
                     product = Product.ETF
                 else:
                     product = Product.INDEX
+                    vol_step = 1
             elif xt_exchange == "BJ":
                 product = Product.EQUITY
 
@@ -316,6 +342,8 @@ class XtMdApi:
                 product=product,
                 size=data["VolumeMultiple"],
                 pricetick=data["PriceTick"],
+                min_volume=vol_step,
+                volume_step=vol_step,
                 history_data=False,
                 net_position=True,
                 gateway_name=self.gateway.gateway_name,
